#!/usr/bin/env python3

import dbus
import json
from utils import timer
import inspect
from logger import logger
from collections import namedtuple

from Xlib import X
from Xlib.display import Display
from Xlib.ext.xtest import fake_input

from autobahn.asyncio.websocket import WebSocketServerProtocol, WebSocketServerFactory
import asyncio


Resolution = namedtuple('Resoltuion', ['width', 'height'])

display = Display()
screen = display.screen()
screen_geometry = screen.root.get_geometry()
host_resolution = Resolution(screen_geometry.width, screen_geometry.height)
password = 'admin'

delimiter = ':'


bus = dbus.SessionBus()
media_player_bus_name = 'org.mpris.MediaPlayer2.{}'
media_player_bus_path = '/org/mpris/MediaPlayer2'
dbus_players = ['vlc', 'clementine']


class LinuxConnectProtocol(WebSocketServerProtocol):
    @timer
    def __init__(self):
        self.remote_resolution = Resolution(0, 0)

        self.current_media_player = 'vlc'

        self.callbacks = {}

        base_message_function_name = 'handle_message_'
        for function_name, function in inspect.getmembers(self, inspect.ismethod):
            if function_name.startswith(base_message_function_name):
                message_name = function_name[len(base_message_function_name):]
                self.callbacks[message_name] = function

        WebSocketServerProtocol.__init__(self)

    def onConnect(self, request):
        print('Client connecting: {0}'.format(request.peer))

    def onOpen(self):
        print('WebSocket connection open.')

    def onMessage(self, payload, isBinary):
        message_data = json.loads(payload.decode('utf8'))
        message_type = message_data['message_type']

        func = self.callbacks.get(message_type, self.handle_message_default)
        func(message_data)

    def onClose(self, was_clean, code, reason):
        print('WebSocket connection closed: {}'.format(reason))

    def handle_message_media_player(self, message):
        if message['player'] == 'mpv':
            self.handle_message_media_player_mpv(message)
        elif message['player'] in dbus_players:
            self.handle_message_media_player_dbus(message)

    def handle_message_media_player_mpv(self, message):
        print('mpv')

    def handle_message_media_player_dbus(self, message):
        try:
            player = message['player']
            bus_name = media_player_bus_name.format(player)
            system_object = dbus.get_object(bus_name, media_player_bus_path)
            # system_iface = dbus.Interface(system_object, media_player_bus_name.format('Player'))
            player_call = system_object.get_dbus_method(message['command'], dbus_interface=media_player_bus_name.format('Player'))
            player_call()
        except dbus.DBusException as ex:
            logger.info('Media Player not open: {}'.format(ex.message))

    def handle_message_mouse_position(self, message):
        try:
            x = int(message['x'])
            y = int(message['y'])
        except ValueError:
            return

        x *= self.mouse_factor_x
        y *= self.mouse_factor_y

        if not (0 < x < host_resolution.width) or not (0 < y < host_resolution.height):
            return

        fake_input(display, X.MotionNotify, x=int(x), y=int(y))
        display.sync()

    def handle_message_mouse_click_position(self, message):
        fake_input(display, X.ButtonPress, 1)
        fake_input(display, X.ButtonRelease, 1)
        display.sync()

    def handle_message_setup(self, message):
        if message['password'] != password:
            # self.sendClose()
            print('wrong password, closing connection')

        width = int(message['width'])
        height = int(message['height'])
        self.remote_resolution = Resolution(width, height)
        self.mouse_factor_x = host_resolution.width / self.remote_resolution.width
        self.mouse_factor_y = host_resolution.height / self.remote_resolution.height

    def handle_message_default(self, message):
        print('Can\'t handle message type: {}'.format(message['message_type']))


class LinuxConnectServerFactory(WebSocketServerFactory):
    def __init__(self, url):
        self.protocol = LinuxConnectProtocol
        WebSocketServerFactory.__init__(self, url)


def run():
    factory = LinuxConnectServerFactory(u'ws://127.0.0.1:9000')

    loop = asyncio.get_event_loop()
    coro = loop.create_server(factory, '0.0.0.0', 9000)
    server = loop.run_until_complete(coro)

    try:
        loop.run_forever()
    except KeyboardInterrupt:
        pass
    finally:
        server.close()
        loop.close()

if __name__ == '__main__':
    run()
